/* Examples and Documentation for
 * SmartCGMS - continuous glucose monitoring and controlling framework
 * https://diabetes.zcu.cz/
 *
 * Copyright (c) since 2018 University of West Bohemia.
 *
 * Contact:
 * diabetes@mail.kiv.zcu.cz
 * Medical Informatics, Department of Computer Science and Engineering
 * Faculty of Applied Sciences, University of West Bohemia
 * Univerzitni 8, 301 00 Pilsen
 * Czech Republic
 * 
 * 
 * Purpose of this software:
 * This software is intended to demonstrate work of the diabetes.zcu.cz research
 * group to other scientists, to complement our published papers. It is strictly
 * prohibited to use this software for diagnosis or treatment of any medical condition,
 * without obtaining all required approvals from respective regulatory bodies.
 *
 * Especially, a diabetic patient is warned that unauthorized use of this software
 * may result into severe injure, including death.
 *
 *
 * Licensing terms:
 * Unless required by applicable law or agreed to in writing, software
 * distributed under these license terms is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include "basic_controller_2.h"
#include "descriptor.h"

#include <scgms/rtl/SolverLib.h>

CBasic_Controller_Model_2::CBasic_Controller_Model_2(scgms::IModel_Parameter_Vector *parameters, scgms::IFilter *output)
	:
	// initialize base class
	CBase_Control_Model(output, basic_controller_model_2::model_id, basic_controller_model_2::bolus_id, basic_controller_model_2::basal_id),
	// convert parameters, so we can work with them using their names
	mParameters(scgms::Convert_Parameters<basic_controller_model_2::TParameters>(parameters, nullptr))
{
}

bool CBasic_Controller_Model_2::On_Event_Execute(scgms::UDevice_Event& event) {

	if (event.is_level_event()) {

		// very simple logic - when the Carb_Intake signal comes (the patient ingested some carbs), emit a bolus of fractional
		// amount given by the carb_ratio controller parameter; but also count in current IG by a factor of ig_factor
		if (event.signal_id() == scgms::signal_Carb_Intake) {
			Emit_Bolus(event.level() * mParameters.carb_ratio + mLast_IG * mParameters.ig_factor, event.device_time());
		}
		// when IG comes, save it
		else if (event.signal_id() == scgms::signal_IG) {
			mLast_IG = event.level();
		}
	}

	return true;
}

void CBasic_Controller_Model_2::Advance_Model(double time_advance_delta) {

	// we have no advanced logic in this controller, so we don't implement anything here

}
