/* Examples and Documentation for
 * SmartCGMS - continuous glucose monitoring and controlling framework
 * https://diabetes.zcu.cz/
 *
 * Copyright (c) since 2018 University of West Bohemia.
 *
 * Contact:
 * diabetes@mail.kiv.zcu.cz
 * Medical Informatics, Department of Computer Science and Engineering
 * Faculty of Applied Sciences, University of West Bohemia
 * Univerzitni 8, 301 00 Pilsen
 * Czech Republic
 * 
 * 
 * Purpose of this software:
 * This software is intended to demonstrate work of the diabetes.zcu.cz research
 * group to other scientists, to complement our published papers. It is strictly
 * prohibited to use this software for diagnosis or treatment of any medical condition,
 * without obtaining all required approvals from respective regulatory bodies.
 *
 * Especially, a diabetic patient is warned that unauthorized use of this software
 * may result into severe injure, including death.
 *
 *
 * Licensing terms:
 * Unless required by applicable law or agreed to in writing, software
 * distributed under these license terms is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include <scgms/lang/dstrings.h>
#include "descriptor.h"
#include "helpers.h"

#include "basic_controller_1.h"
#include "basic_controller_2.h"
////// NOTE OP-E: add more include files as you implement more controllers

void init_entities() {

	////// NOTE OP-E: this block describes a treatment model

	// create treatment model implemented by CBasic_Controller_Model_1 class (defined in basic_controller_1.h and .cpp files)
	Describe_Discrete_Model<CBasic_Controller_Model_1>(
		// ID of the model (defined in descriptor.h)
		basic_controller_model_1::model_id,
		// name of your model
		L"[OP-E] Basic controller I.",
		// parameters of the model
		{
			// name, parameter type (almost always mptDouble), lower bound,  default value, upper bound
			{ L"cr", scgms::NModel_Parameter_Value::mptDouble, 0.01,         0.066,         0.5 },
		},
		// signals of the model - we always describe signals, so we are able to visualize them
		{
			{ basic_controller_model_1::bolus_id, L"[OP-E] Bolus I.", dsU, scgms::NSignal_Unit::U_insulin, 0xFF8800FF, 0xFF8800FF, scgms::NSignal_Visualization::mark, scgms::NSignal_Mark::triangle, nullptr, 1.0 },
			{ basic_controller_model_1::basal_id, L"[OP-E] Basal I.", dsU_per_Hr, scgms::NSignal_Unit::U_per_Hr, 0xFF0088FF, 0xFF0088FF, scgms::NSignal_Visualization::step, scgms::NSignal_Mark::none, nullptr, 1.0 }
		}
	);

	// create treatment model implemented by CBasic_Controller_Model_2 class (defined in basic_controller_2.h and .cpp files)
	Describe_Discrete_Model<CBasic_Controller_Model_2>(
		// ID of the model (defined in descriptor.h)
		basic_controller_model_2::model_id,
		// name of your model
		L"[OP-E] Basic controller II.",
		// parameters of the model
		{
			// name,  parameter type (almost always mptDouble), lower bound,  default value, upper bound
			{ L"cr",  scgms::NModel_Parameter_Value::mptDouble, 0.01,         0.066,         0.5 },
			{ L"igf", scgms::NModel_Parameter_Value::mptDouble, 0.05,         0.1,           0.4 }
		},
		// signals of the model - we always describe signals, so we are able to visualize them
		{
			{ basic_controller_model_2::bolus_id, L"[OP-E] Bolus II.", dsU, scgms::NSignal_Unit::U_insulin, 0xFF8800FF, 0xFF8800FF, scgms::NSignal_Visualization::mark, scgms::NSignal_Mark::triangle, nullptr, 1.0 },
			{ basic_controller_model_2::basal_id, L"[OP-E] Basal II.", dsU_per_Hr, scgms::NSignal_Unit::U_per_Hr, 0xFF0088FF, 0xFF0088FF, scgms::NSignal_Visualization::step, scgms::NSignal_Mark::none, nullptr, 1.0 }
		}
	);

	////// NOTE OP-E: when defining parameters, define some reasonable values for lower and upper bound - these are the bounds for the parameter. As the solver searches for best value of this parameter, it always chooses 
	// a value within these bounds; the "default value" is an initial value, that is given to the parameter when no value was found yet

	////// NOTE OP-E: when defining a new model, try to follow the same pattern as above
}
